/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.osgi.blueprint.parser.xb;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlType;

import org.jboss.osgi.blueprint.BlueprintContext;

// $Id: TServicePropertyEntry.java 91348 2009-07-16 15:54:22Z thomas.diesler@jboss.com $

/**
 * TservicePropertyEntry is an entry value used for the <service-properties> element. 
 * This does not allow a child <key> element and there are no key-ref or value-ref
 * attributes.
 * 
 * <xsd:complexType name="TservicePropertyEntry">
 *   <xsd:sequence>
 *     <xsd:group ref="Gvalue" minOccurs="0" />
 *   </xsd:sequence>
 *   <xsd:attribute name="key" type="TstringValue" use="required" />
 *   <xsd:attribute name="value" type="TstringValue" />
 * </xsd:complexType>
 * 
 * @author thomas.diesler@jboss.com
 * @since 13-May-2009
 */
@XmlType(name = "TservicePropertyEntry", namespace = BlueprintContext.XMLNS_BLUEPRINT)
public class TServicePropertyEntry extends GValue
{
   protected String key;
   protected String value;

   @XmlAttribute(required = true)
   public String getKey()
   {
      return key;
   }

   public void setKey(String key)
   {
      this.key = key;
   }

   @XmlAttribute
   public String getValue()
   {
      return value;
   }

   public void setValue(String value)
   {
      this.value = value;
   }
}
