/*
* JBoss, Home of Professional Open Source
* Copyright 2009, Red Hat Middleware LLC, and individual contributors
* as indicated by the @author tags. See the copyright.txt file in the
* distribution for a full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.osgi.framework.filter.model;

/**
 * ApproximateOperation.
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 1.1 $
 */
public class ApproximateOperation extends CompareOperation
{
   /**
    * Create a new ApproximateOperation.
    * 
    * @param left the attribute
    * @param right the value
    */
   public ApproximateOperation(String left, String right)
   {
      super(left, right);
   }

   @Override
   @SuppressWarnings("unchecked")
   protected boolean compare(Comparable lhs, Object rhs)
   {
      return isApproximate(lhs, rhs);
   }

   @Override
   protected boolean equals(Object lhs, Object rhs)
   {
      return isApproximate(lhs, rhs);
   }

   /**
    * Test whether two values are approximately equal
    * 
    * @param lhs the left hand side
    * @param rhs the right hand side
    * @return true when approximate
    */
   protected boolean isApproximate(Object lhs, Object rhs)
   {
      // TODO approximate
      return lhs.equals(rhs);
   }
   
   @Override
   public void appendOperation(StringBuilder builder)
   {
      builder.append("~=");
   }
}
